/* "eqcs_bitset.h"
 * Copyright (C) 1999 Peter Belkner <pbelkner@berlin.snafu.de>
 *
 * This file is part of the eqcs library. This library is free
 * software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License as published by the Free Software
 * Foundation; either version 2, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; see the file COPYING.  If not, write to the
 * Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifndef	_EQCS_BITSET_H_
#ifdef __GNUG__
#pragma interface
#endif
#define	_EQCS_BITSET_H_

#include "eqcs_iomanip.h"
#include "eqcs_handle.h"

class EqcsBitsetRep;

/*****************************************************************************\
*	bitset																	  *
\*****************************************************************************/

class EqcsBitset: public EqcsHandle {
public:
	typedef EqcsHandle base_t;
	typedef EqcsBitsetRep rep_t;
	typedef EqcsOManip0<EqcsBitset> omanip0_t;
	typedef EqcsOManip1<EqcsBitset, int> omanip1_t;
	typedef EqcsOManip2<EqcsBitset, int, int> omanip2_t;

	EqcsBitset(rep_t *rep);
	EqcsBitset();
	explicit EqcsBitset(int nbits);

	int nbits() const;

	void set(int i) const;
	void reset(int i) const;
	bool test(int i) const;

	EqcsBitset operator~() const;
	EqcsBitset &operator&=(const EqcsBitset &bitset);
	EqcsBitset &operator|=(const EqcsBitset &bitset);
	EqcsBitset &operator<<=(int i);
	EqcsBitset &operator>>=(int i);

	ostream &put_ket(ostream &os) const;
	ostream &put_ket(ostream &os, int nbits) const;
	ostream &put_bits(ostream &os, int nregs, int regwidth) const;
	ostream &put_nums(ostream &os, int nregs, int regwidth) const;

	static int leftmost(unsigned long bits);
	static EqcsBitset bits2bitset(unsigned long bits);

	// Some technical uitilities.
	inline rep_t *operator->() const
		{ return (rep_t *)EqcsHandle::rep(); }
	inline rep_t *rep() const
		{ return (rep_t *)EqcsHandle::rep(); }
	inline operator rep_t *() const
		{ return (rep_t *)EqcsHandle::rep(); }
};

bool operator==(const EqcsBitset &l, const EqcsBitset &r);
bool operator!=(const EqcsBitset &l, const EqcsBitset &r);
bool operator<(const EqcsBitset &l, const EqcsBitset &r);
bool operator<=(const EqcsBitset &l, const EqcsBitset &r);
bool operator>=(const EqcsBitset &l, const EqcsBitset &r);
bool operator>(const EqcsBitset &l, const EqcsBitset &r);

EqcsBitset operator&(const EqcsBitset &l, const EqcsBitset &r);
EqcsBitset operator|(const EqcsBitset &l, const EqcsBitset &r);
EqcsBitset operator>>(const EqcsBitset &l, int i);
EqcsBitset operator<<(const EqcsBitset &l, int i);

ostream &operator<<(ostream &os, const EqcsBitset &bitset);
EqcsBitset::omanip0_t ket(const EqcsBitset &bitset);
EqcsBitset::omanip1_t ket(const EqcsBitset &bitset, int nbits);
EqcsBitset::omanip2_t bits(const EqcsBitset &bitset, int nregs, int regwidth);
EqcsBitset::omanip2_t nums(const EqcsBitset &bitset, int nregs, int regwidth);

#endif	//	_EQCS_BITSET_H_
