/* "eqcs_iomanip.h"
 * Copyright (C) 1999 Peter Belkner <pbelkner@berlin.snafu.de>
 *
 * This file is part of the eqcs library. This library is free
 * software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License as published by the Free Software
 * Foundation; either version 2, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; see the file COPYING.  If not, write to the
 * Free Software Foundation, 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifndef	_EQCS_IOMANIP_H_
#ifdef __GNUG__
#pragma interface
#endif
#define	_EQCS_IOMANIP_H_

#include <iostream>

using namespace std;

/*****************************************************************************\
*	zero-argument manipulator												  *
\*****************************************************************************/

template <class T>
class EqcsOManip0 {
public:
	typedef ostream &(T::*put_t)(ostream &os) const;

	EqcsOManip0(const T &obj, put_t put) : _obj(obj)
		{ _put = put; }
	friend ostream &operator<<(ostream &os, const EqcsOManip0 &m)
		{ return (m._obj.*m._put)(os); }
private:
	T			_obj;
	put_t		_put;
};

/*****************************************************************************\
*	one-argument manipulator												  *
\*****************************************************************************/

template <class T, class A>
class EqcsOManip1 {
public:
	typedef ostream &(T::*put_t)(ostream &os, A a) const;

	EqcsOManip1(const T &obj, put_t put, A a) : _obj(obj), _a(a)
		{ _put = put; }
	friend ostream &operator<<(ostream &os, const EqcsOManip1 &m)
		{ return (m._obj.*m._put)(os, m._a); }
private:
	T			_obj;
	put_t		_put;
	A			_a;
};

/*****************************************************************************\
*	two-argument manipulator												  *
\*****************************************************************************/

template <class T, class A1, class A2>
class EqcsOManip2 {
public:
	typedef ostream &(T::*put_t)(ostream &os, A1 a1, A2 a2) const;

	EqcsOManip2(const T &obj, put_t put, A1 a1, A2 a2) :
		_obj(obj), _a1(a1), _a2(a2)
		{ _put = put; }
	friend ostream &operator<<(ostream &os, const EqcsOManip2 &m)
		{ return (m._obj.*m._put)(os, m._a1, m._a2); }
private:
	T			_obj;
	put_t		_put;
	A1			_a1;
	A2			_a2;
};

#endif	//	_EQCS_IOMANIP_H_
